/*
 * Avalon memory-mapped peripheral that generates VGA
 *
 * Stephen A. Edwards
 * Columbia University
 */

module mixer(
	input logic clk,
	input logic reset,
	input logic [31:0]  writedata,
	input logic 	write,

	input logic read,
	input logic chipselect,
	input logic [1:0] address, 

	output logic [31:0] readdata,

	output logic [7:0] VGA_R, VGA_G, VGA_B,
	output logic	VGA_CLK, VGA_HS, VGA_VS, 
			VGA_BLANK_n,
	output logic VGA_SYNC_n,

	input left_ready, right_ready,
	output [15:0] left_data, right_data,
	output logic left_valid, right_valid,

	output logic irq);

	logic [31:0] fifo_stream;
	logic async_clr = 0;
	logic read_req = 0;
	logic empty;
	logic full;
	logic almost_empty;
	logic almost_full;
	logic [9:0] used_words;
	reg [11:0] counter;

	//store song samples
	fifo_interrupt song_data1(.aclr(async_clr), .clock(clk), .data(writedata),
		.rdreq(read_req), .wrreq(chipselect && write),
		.almost_empty(almost_empty), .almost_full(almost_full),
		.empty(empty), .full(full), .q(fifo_stream), .usedw(used_words));

	enum logic [1:0] {EMPTY, FULL, FILLING} state;	
   	always_ff @(posedge clk) begin
		if (reset) begin
			counter <= 0;
			async_clr <= 1'b1;
			left_valid <= 0;
			right_valid <= 0;
			read_req <= 0;
			irq <= 1;
			state <= EMPTY;
		end else begin 
			//handle writes from software
			case (state)
			EMPTY: begin
				readdata <= 16'he;
				if (chipselect && read) begin
					irq <= 0;
				end
				if (!almost_empty) begin
					state <= FILLING;
				end
			end
			FILLING: begin
				irq <= 0;
				if (almost_empty) begin
					irq <= 1;
					state <= EMPTY;
				end else if (almost_full) begin
					irq <= 1;
					state <= FULL;
				end
			end
			FULL: begin
				readdata <= 16'hf;
				if (chipselect && read) begin
					irq <= 0;
				end
				if (!almost_full) begin
					state <= FILLING;
				end
			end
			default: ;
			endcase
			//send data to fft block
			if (!empty && left_ready == 1 && right_ready == 1) begin
				left_valid <= 1;
				right_valid <= 1;
				read_req <= 1;
				left_data <= (fifo_stream[31:16]);
				right_data <= (fifo_stream[15:0]);
				async_clr <= 1'b0;
	       	end else begin
				left_valid <= 0;
				right_valid <= 0;
				read_req <= 0;
				async_clr <= 1'b0;
			end
		end
	end
endmodule
